#ifndef __I__ExpansionWave__H__
#define __I__ExpansionWave__H__

// constans

#define NX 140	//65
#define NY 100	//41

#ifndef M_PI
#define M_PI 3.141592654f
#endif

#define DTR (M_PI/180.0f)		// degree to radian
#define RTD (180.0f/M_PI)		// radian to degree

#define Courant 0.9f			// Courant number 
#define Cy 0.6f				// coefficient for artifical viscosity

#define gamma 1.4f			// ideal gas
#define M_R 287.0f			// ideal gas constant from sample calculations in a book

#define M_E 10				// [m] - expansion corner position
#define H0 40				// [m] - height before expansion corner


// structures

typedef struct GridPoint GridPoint;
struct GridPoint
{
	// physics

	double y;
	double ys;

	double u;
	double v;
	double rho;
	double p;
	double t;
	double m;
	double miu;			// sin(1/m)

	// physics predicted values
	double up;
	double vp;
	double rhop;
	double pp;
//	double t;
//	double m;
//	double miu;			// sin(1/m)


	// metrics
	double dmidx;
	double mi;

	double dk;			// delta ksi

	// flux

	double F1;
	double F2;
	double F3;
	double F4;

	double G1;
	double G2;
	double G3;
	double G4;

	double dF1dk;
	double dF2dk;
	double dF3dk;
	double dF4dk;

	double dG1dk;
	double dG2dk;
	double dG3dk;
	double dG4dk;

	// flux predictor

	double F1p;
	double F2p;
	double F3p;
	double F4p;

	double G1p;
	double G2p;
	double G3p;
	double G4p;

	double dF1dkp;
	double dF2dkp;
	double dF3dkp;
	double dF4dkp;

	// averages

	double dF1dkav;
	double dF2dkav;
	double dF3dkav;
	double dF4dkav;

	// artifical viscosity

	double SF1;
	double SF2;
	double SF3;
	double SF4;

	// predicted artifical viscosity

	double SF1p;
	double SF2p;
	double SF3p;
	double SF4p;
};


// functions

void initial_values(double u, double v, double rho, double p, double t, double m);
void encode(int i, int j);
double h(double x);
double ys(double x);
double y(double x, int j);

double calculate_dksi(double x, int i);

void predictor_step_pre(double x, int i);
void predictor_step_1(double x, int i, int j, double dmi, double dk);
void predictor_decode_rho(int i, int j);
void predictor_encode_g(int i, int j);
void corrector_step_1(double x,int i,int j,double dmi,double dk);
void corrector_step_2(int i,int j,double dk);
void corrector_encode_physcis(int i, int j);
void abbett(double x, int i);


#endif